//B4J WebSockets ABMaterial client library v2.11

(function (global, factory) {
    if (typeof define === 'function' && define.amd) {
        define([], factory);
    } else if (typeof module !== 'undefined' && module.exports){
        module.exports = factory();
    } else {
        global.ReconnectingWebSocket = factory();
    }
})(this, function () {

    if (!('WebSocket' in window)) {
        return;
    }

    function ReconnectingWebSocket(url, protocols, options) {

        // Default settings
        var settings = {

            /** Whether this instance should log debug messages. */
            debug: false,

            /** Whether or not the websocket should attempt to connect immediately upon instantiation. */
            automaticOpen: true,

            /** The number of milliseconds to delay before attempting to reconnect. */
            reconnectInterval: 1000,
            /** The maximum number of milliseconds to delay a reconnection attempt. */
            maxReconnectInterval: 30000,
            /** The rate of increase of the reconnect delay. Allows reconnect attempts to back off when problems persist. */
            reconnectDecay: 1.5,

            /** The maximum time in milliseconds to wait for a connection to succeed before closing and retrying. */
            timeoutInterval: 2000,

            /** The maximum number of reconnection attempts to make. Unlimited if null. */
            maxReconnectAttempts: null,

            /** The binary type, possible values 'blob' or 'arraybuffer', default 'blob'. */
            binaryType: 'blob'
        }
        if (!options) { options = {}; }

        // Overwrite and define settings with options if they exist.
        for (var key in settings) {
            if (typeof options[key] !== 'undefined') {
                this[key] = options[key];
            } else {
                this[key] = settings[key];
            }
        }

        // These should be treated as read-only properties

        /** The URL as resolved by the constructor. This is always an absolute URL. Read only. */
        this.url = url;

        /** The number of attempted reconnects since starting, or the last successful connection. Read only. */
        this.reconnectAttempts = 0;

        /**
         * The current state of the connection.
         * Can be one of: WebSocket.CONNECTING, WebSocket.OPEN, WebSocket.CLOSING, WebSocket.CLOSED
         * Read only.
         */
        this.readyState = WebSocket.CONNECTING;

        /**
         * A string indicating the name of the sub-protocol the server selected; this will be one of
         * the strings specified in the protocols parameter when creating the WebSocket object.
         * Read only.
         */
        this.protocol = null;

        // Private state variables

        var self = this;
        var ws;
        var forcedClose = false;
        var timedOut = false;
        var eventTarget = document.createElement('div');

        // Wire up "on*" properties as event handlers

        eventTarget.addEventListener('open',       function(event) { self.onopen(event); });
        eventTarget.addEventListener('close',      function(event) { self.onclose(event); });
        eventTarget.addEventListener('connecting', function(event) { self.onconnecting(event); });
        eventTarget.addEventListener('message',    function(event) { self.onmessage(event); });
        eventTarget.addEventListener('error',      function(event) { self.onerror(event); });

        // Expose the API required by EventTarget

        this.addEventListener = eventTarget.addEventListener.bind(eventTarget);
        this.removeEventListener = eventTarget.removeEventListener.bind(eventTarget);
        this.dispatchEvent = eventTarget.dispatchEvent.bind(eventTarget);

        /**
         * This function generates an event that is compatible with standard
         * compliant browsers and IE9 - IE11
         *
         * This will prevent the error:
         * Object doesn't support this action
         *
         * http://stackoverflow.com/questions/19345392/why-arent-my-parameters-getting-passed-through-to-a-dispatched-event/19345563#19345563
         * @param s String The name that the event should use
         * @param args Object an optional object that the event will use
         */
        function generateEvent(s, args) {
        	var evt = document.createEvent("CustomEvent");
        	evt.initCustomEvent(s, false, false, args);
        	return evt;
        };

        this.open = function (reconnectAttempt) {
            ws = new WebSocket(self.url, protocols || []);
            ws.binaryType = this.binaryType;

            if (reconnectAttempt) {
                if (this.maxReconnectAttempts && this.reconnectAttempts > this.maxReconnectAttempts) {
                    return;
                }
            } else {
                eventTarget.dispatchEvent(generateEvent('connecting'));
                this.reconnectAttempts = 0;
            }

            if (self.debug || ReconnectingWebSocket.debugAll) {
                console.debug('ReconnectingWebSocket', 'attempt-connect', self.url);
            }

            var localWs = ws;
            var timeout = setTimeout(function() {
                if (self.debug || ReconnectingWebSocket.debugAll) {
                    console.debug('ReconnectingWebSocket', 'connection-timeout', self.url);
                }
                timedOut = true;
                localWs.close();
                timedOut = false;
            }, self.timeoutInterval);

            ws.onopen = function(event) {
                clearTimeout(timeout);
                if (self.debug || ReconnectingWebSocket.debugAll) {
                    console.debug('ReconnectingWebSocket', 'onopen', self.url);
                }
                self.protocol = ws.protocol;
                self.readyState = WebSocket.OPEN;
                self.reconnectAttempts = 0;
                var e = generateEvent('open');
                e.isReconnect = reconnectAttempt;
                reconnectAttempt = false;
                eventTarget.dispatchEvent(e);
            };

            ws.onclose = function(event) {
                clearTimeout(timeout);
                ws = null;
                if (forcedClose) {
                    self.readyState = WebSocket.CLOSED;
                    eventTarget.dispatchEvent(generateEvent('close'));
                } else {
                    self.readyState = WebSocket.CONNECTING;
                    var e = generateEvent('connecting');
                    e.code = event.code;
                    e.reason = event.reason;
                    e.wasClean = event.wasClean;
                    eventTarget.dispatchEvent(e);
                    if (!reconnectAttempt && !timedOut) {
                        if (self.debug || ReconnectingWebSocket.debugAll) {
                            console.debug('ReconnectingWebSocket', 'onclose', self.url);
                        }
                        eventTarget.dispatchEvent(generateEvent('close'));
                    }

                    var timeout = self.reconnectInterval * Math.pow(self.reconnectDecay, self.reconnectAttempts);
                    setTimeout(function() {
                        self.reconnectAttempts++;
                        self.open(true);
                    }, timeout > self.maxReconnectInterval ? self.maxReconnectInterval : timeout);
                }
            };
            ws.onmessage = function(event) {
                if (self.debug || ReconnectingWebSocket.debugAll) {
                    console.debug('ReconnectingWebSocket', 'onmessage', self.url, event.data);
                }
                var e = generateEvent('message');
                e.data = event.data;
                eventTarget.dispatchEvent(e);
            };
            ws.onerror = function(event) {
                if (self.debug || ReconnectingWebSocket.debugAll) {
                    console.debug('ReconnectingWebSocket', 'onerror', self.url, event);
                }
                eventTarget.dispatchEvent(generateEvent('error'));
            };
        }

        // Whether or not to create a websocket upon instantiation
        if (this.automaticOpen == true) {
            this.open(false);
        }

        /**
         * Transmits data to the server over the WebSocket connection.
         *
         * @param data a text string, ArrayBuffer or Blob to send to the server.
         */
        this.send = function(data) {
            if (ws) {
                if (self.debug || ReconnectingWebSocket.debugAll) {
                    console.debug('ReconnectingWebSocket', 'send', self.url, data);
                }
                return ws.send(data);
            } else {
                throw 'INVALID_STATE_ERR : Pausing to reconnect websocket';
            }
        };

        /**
         * Closes the WebSocket connection or connection attempt, if any.
         * If the connection is already CLOSED, this method does nothing.
         */
        this.close = function(code, reason) {
            // Default CLOSE_NORMAL code
            if (typeof code == 'undefined') {
                code = 1000;
            }
            forcedClose = true;
            if (ws) {
                ws.close(code, reason);
            }
        };

        /**
         * Additional public API method to refresh the connection if still open (close, re-open).
         * For example, if the app suspects bad data / missed heart beats, it can try to refresh.
         */
        this.refresh = function() {
            if (ws) {
                ws.close();
            }
        };
    }

    /**
     * An event listener to be called when the WebSocket connection's readyState changes to OPEN;
     * this indicates that the connection is ready to send and receive data.
     */
    ReconnectingWebSocket.prototype.onopen = function(event) {};
    /** An event listener to be called when the WebSocket connection's readyState changes to CLOSED. */
    ReconnectingWebSocket.prototype.onclose = function(event) {};
    /** An event listener to be called when a connection begins being attempted. */
    ReconnectingWebSocket.prototype.onconnecting = function(event) {};
    /** An event listener to be called when a message is received from the server. */
    ReconnectingWebSocket.prototype.onmessage = function(event) {};
    /** An event listener to be called when an error occurs. */
    ReconnectingWebSocket.prototype.onerror = function(event) {};

    /**
     * Whether all instances of ReconnectingWebSocket should log debug messages.
     * Setting this to true is the equivalent of setting all instances of ReconnectingWebSocket.debug to true.
     */
    ReconnectingWebSocket.debugAll = false;

    ReconnectingWebSocket.CONNECTING = WebSocket.CONNECTING;
    ReconnectingWebSocket.OPEN = WebSocket.OPEN;
    ReconnectingWebSocket.CLOSING = WebSocket.CLOSING;
    ReconnectingWebSocket.CLOSED = WebSocket.CLOSED;

    return ReconnectingWebSocket;
});

var hasEvents=false;
// modified version Alain Bailleul for ABMaterial with ReconnectingWebSocket
function hasClass(el, className) {
  if (el.classList)
    return el.classList.contains(className)
  else
    return !!el.className.match(new RegExp('(\\s|^)' + className + '(\\s|$)'))
};

function addClass(el, className) {
  if (el.classList)
    el.classList.add(className)
  else if (!hasClass(el, className)) el.className += " " + className
};

function removeClass(el, className) {
  if (el.classList)
    el.classList.remove(className)
  else if (hasClass(el, className)) {
    var reg = new RegExp('(\\s|^)' + className + '(\\s|$)')
    el.className=el.className.replace(reg, ' ')
  }
};

if (typeof Object.assign != 'function') {
  (function () {
    Object.assign = function (target) {
      'use strict';
      // We must check against these specific cases.
      if (target === undefined || target === null) {
        throw new TypeError('Cannot convert undefined or null to object');
      }

      var output = Object(target);
      for (var index = 1; index < arguments.length; index++) {
        var source = arguments[index];
        if (source !== undefined && source !== null) {
          for (var nextKey in source) {
            if (source.hasOwnProperty(nextKey)) {
              output[nextKey] = source[nextKey];
            }
          }
        }
      }
      return output;
    };
  })();
};

(function(global) {

  if (typeof global.CustomEvent === 'function') return

  function CustomEvent(event, params) {
    params = params || {
        bubbles: false,
        cancelable: false,
        detail: undefined
    }

    var evt
    if (typeof document !== 'undefined') {
      evt = document.createEvent('CustomEvent')
      evt.initCustomEvent(event, params.bubbles, params.cancelable, params.detail)
    } else {
      evt = {
        type: event,
        detail: params.detail,
        bubbles: false,
        cancelable: false,
        preventDefault: function() {},
        stopPropagation: function() {}
      }
    }

    return evt
  }

  if (global.Event) {
    CustomEvent.prototype = global.Event.prototype
  }

  global.CustomEvent = CustomEvent
})(this);


(function(factory, global) {
  if (typeof define === 'function' && define.amd) {
    define(function() {
      return factory(global, navigator)
    })
  } else if (typeof exports === 'object' && typeof module === 'object') {
    module.exports = factory(global, navigator)
  } else {
    // mock the navigator object when under test since `navigator.onLine` is read only
    global.RobustWebSocket = factory(global, typeof Mocha !== 'undefined' ? Mocha : navigator)
  }
})(function(global, navigator) {

  var RobustWebSocket = function(url, protocols, userOptions) {
    var realWs = { close: function() {} },
        connectTimeout,
        self = this,
        attempts = 0,
        reconnects = -1,
        reconnectWhenOnlineAgain = false,
        explicitlyClosed = false,
        pendingReconnect,
        opts = Object.assign({},
          RobustWebSocket.defaultOptions,
          typeof userOptions === 'function' ? { shouldReconnect: userOptions } : userOptions
        )

    if (typeof opts.timeout !== 'number') {
      throw new Error('timeout must be the number of milliseconds to timeout a connection attempt')
    }

    if (typeof opts.shouldReconnect !== 'function') {
      throw new Error('shouldReconnect must be a function that returns the number of milliseconds to wait for a reconnect attempt, or null or undefined to not reconnect.')
    }

    ['bufferedAmount', 'url', 'readyState', 'protocol', 'extensions'].forEach(function(readOnlyProp) {
      Object.defineProperty(self, readOnlyProp, {
        get: function() { return realWs[readOnlyProp] }
      })
    })

    function clearPendingReconnectIfNeeded() {
      if (pendingReconnect) {
        pendingReconnect = null
        clearTimeout(pendingReconnect)
      }
    }

    var ononline = function(event) {	
		//console.log("online");
	  if (reconnectWhenOnlineAgain) {
        clearPendingReconnectIfNeeded()
        reconnect(event)
      }
    },
    onoffline = function() {		
		//console.log("offline");
	  reconnectWhenOnlineAgain = true
      realWs.close(1000)
    },
    connectivityEventsAttached = false

    function detachConnectivityEvents() {
      if (connectivityEventsAttached) {
        global.removeEventListener('online', ononline);
        global.removeEventListener('offline', onoffline);
        connectivityEventsAttached = false;
		hasEvents=false;		
      }
    }

    function attachConnectivityEvents() {
	  if (!connectivityEventsAttached) {
        global.addEventListener('online', ononline);
        global.addEventListener('offline', onoffline);
        connectivityEventsAttached = true;
		hasEvents=true;		
      }
    }

    self.send = function() {
      return realWs.send.apply(realWs, arguments)
    }

    self.close = function(code, reason) {
      if (typeof code !== 'number') {
        reason = code
        code = 1000
      }

      clearPendingReconnectIfNeeded()
      reconnectWhenOnlineAgain = false
      explicitlyClosed = true
      detachConnectivityEvents()

      return realWs.close(code, reason)
    }

    self.open = function() {
      if (realWs.readyState !== WebSocket.OPEN && realWs.readyState !== WebSocket.CONNECTING) {
        clearPendingReconnectIfNeeded()
        reconnectWhenOnlineAgain = false
        explicitlyClosed = false

        newWebSocket()
      }
    }
	
	self.startReconnect = function(event) {
		if (reconnectWhenOnlineAgain) {
			clearPendingReconnectIfNeeded()
			reconnect(event)
		}
	}

    function reconnect(event) {
      if (event.code === 1000 || explicitlyClosed) {
	    console.log("Stop reconnecting: " + event.code + "," + explicitlyClosed);
		attempts = 0
		var indicator = document.getElementById('pageconnectedindicator');
		if (indicator === null) {
			var randomNum = Math.round(Math.random() * 1000000);
			RunToastNoConnection(randomNum);
		}
        return
      }
      if (navigator.onLine === false) {
        reconnectWhenOnlineAgain = true
        return
      }

      var delay = opts.shouldReconnect(event, self)
	  if (typeof delay === 'number') {
        pendingReconnect = setTimeout(newWebSocket, delay)
      }
    }

    Object.defineProperty(self, 'listeners', {
      value: {
        open: [function(event) {
          if (connectTimeout) {
            clearTimeout(connectTimeout)
            connectTimeout = null
          }
          event.reconnects = ++reconnects
          event.attempts = attempts
          attempts = 0
          reconnectWhenOnlineAgain = false
        }],
        close: [reconnect]
      }
    })
	
    Object.defineProperty(self, 'attempts', {
      get: function() { return attempts },
      enumerable: true
    })

    Object.defineProperty(self, 'reconnects', {
      get: function() { return reconnects },
      enumerable: true
    })

    function newWebSocket() {
      pendingReconnect = null
      realWs = new WebSocket(url, protocols)
      realWs.binaryType = self.binaryType

      attempts++
      self.dispatchEvent(Object.assign(new CustomEvent('connecting'), {
        attempts: attempts,
        reconnects: reconnects
      }))

      connectTimeout = setTimeout(function() {
        connectTimeout = null
        detachConnectivityEvents()
        self.dispatchEvent(Object.assign(new CustomEvent('timeout'), {
          attempts: attempts,
          reconnects: reconnects
        }))
      }, opts.timeout)

      ;['open', 'close', 'message', 'error'].forEach(function(stdEvent) {
        realWs.addEventListener(stdEvent, function(event) {
          self.dispatchEvent(event)

          var cb = self['on' + stdEvent]
          if (typeof cb === 'function') {
            return cb.apply(self, arguments)
          }
        })
      })

      attachConnectivityEvents()
    }

    if (opts.automaticOpen) {
      newWebSocket()
    }
  }

  RobustWebSocket.defaultOptions = {
    // the time to wait before a successful connection
    // before the attempt is considered to have timed out
    timeout: 1000,
    // Given a CloseEvent or OnlineEvent and the RobustWebSocket state,
    // should a reconnect be attempted? Return the number of milliseconds to wait
    // to reconnect (or null or undefined to not), rather than true or false
    shouldReconnect: function(event, ws) {
	  if (event.type === 'online') return 0;	
      if (event.code === 1008 || event.code === 1011) return
      //return [0, 3000, 10000][ws.attempts]
	  return 1000
    },

    // Create and connect the WebSocket when the instance is instantiated.
    // Defaults to true to match standard WebSocket behavior
    automaticOpen: true
  }

  RobustWebSocket.prototype.binaryType = 'blob'

  // Taken from MDN https://developer.mozilla.org/en-US/docs/Web/API/EventTarget
  RobustWebSocket.prototype.addEventListener = function(type, callback) {
    if (!(type in this.listeners)) {
      this.listeners[type] = []
    }
    this.listeners[type].push(callback)
  }

  RobustWebSocket.prototype.removeEventListener = function(type, callback) {
    if (!(type in this.listeners)) {
      return
    }
    var stack = this.listeners[type]
    for (var i = 0, l = stack.length; i < l; i++) {
      if (stack[i] === callback) {
        stack.splice(i, 1)
        return
      }
    }
  }

  RobustWebSocket.prototype.dispatchEvent = function(event) {
    if (!(event.type in this.listeners)) {
      return
    }
    event.currentTarget = this
    var stack = this.listeners[event.type]
    for (var i = 0, l = stack.length; i < l; i++) {
      stack[i].call(this, event)
    }
  }

  return RobustWebSocket
}, this)

"use strict";
var b4j_ws;
var b4j_closeMessage = false;
var readyTimeout;
var PageReadyAlreadySend=false;
var mySessionId="";
var reconnectTimeout;
var isAndroidNative = "";
var WebSocketType;
//only called as a result of a server request that is waiting for result.
//this method should not be called in any other case.
function b4j_sendData(data) {
    b4j_ws.send(JSON.stringify({type: "data", data: data}));
}
function b4j_raiseEvent(eventName, parameters) {	
	var indicator = document.getElementById('pageconnectedindicator');
	
	try {
        if (b4j_ws.readyState !== 1) {
            if (b4j_closeMessage === false) {
                //window.console.error("connection is closed.");
				if (typeof RobustWebSocket === 'undefined') {
					//window.alert("Server is currently not available.");
					window.console.error("Server is currently not available.");	
				} else {
					//window.alert("Connection is closed. Please refresh the page to reconnect.");
					window.console.error("Connection is closed. Trying to reconnect.");						
				}
                b4j_closeMessage = true;				
            }			
			if (indicator === null) {
				var randomNum = Math.round(Math.random() * 1000000);
				RunToastNoConnection(randomNum);
			}
        } else {
			b4j_closeMessage = false;
			//console.log("sending: " + parameters.eventname);
            b4j_ws.send(JSON.stringify({type: "event", event: eventName, params: parameters}));		
			if (parameters.eventname=='page_ready') {
				PageReadyAlreadySend=true;
				if (readyTimeout) {
					//clearTimeout(readyTimeout);
					clearInterval(readyTimeout);
				}	
				
			}
        }
    } catch (e) {
		window.console.error(e);
		if (indicator === null) {
			var randomNum = Math.round(Math.random() * 1000000);
			RunToastNoConnection(randomNum);
		}
    }
	
}
function b4j_addEvent(selector, event, eventName, preventDefault) {
    var obj = $(selector);
    if (obj.length > 0) {
        obj.on(event, function (e) {
            if (preventDefault) {
                e.preventDefault();
                e.stopPropagation();
            }
            if (e.target.id) {
                b4j_raiseEvent(eventName, {which: e.which, target: e.target.id, pageX: e.pageX, pageY: e.pageY, metaKey: e.metaKey});				
            } else {
                b4j_raiseEvent(eventName, {which: e.which, target: selector, pageX: e.pageX, pageY: e.pageY, metaKey: e.metaKey});
            }
        });
    }
}
function b4j_addAutomaticEvents(data) {	
    $.each(data, function (index, value) {		
		b4j_addEvent("#" + value.id, value.event, value.id + "_" + value.event, true);		
    });
	
}
function b4j_runFunction(func, params) {
    //console.log(func);
	return window[func].apply(null, params);
}

function b4j_eval(params, script) {
	//console.log(script);
	var f = new Function(script);
    return f.apply(null, params);
}

function b4j_connect(absolutePath) {	
	//if (Cookies.enabled) {
	//	console.log(Cookies.get("JSESSIONID"));
	//}
    if (typeof WebSocket === 'undefined') {
        window.alert("WebSockets are not supported by your browser.");
        return;
    }
    var l = window.location, fullpath;
	if  (l.port != 0) {
        fullpath = ((l.protocol === "https:") ? "wss://" : "ws://") + l.hostname + ":" + l.port + absolutePath + l.search;
    } else {
        fullpath = ((l.protocol === "https:") ? "wss://" : "ws://") + l.hostname  + absolutePath + l.search;
    }
	
	var info = platform.parse(navigator.userAgent);
	var sTest = "" + info.os;
	var isAndroidNative = ((sTest.indexOf('Android')>-1) && (sTest < 'Android 6'));
	
	if (typeof RobustWebSocket === 'undefined') {
		console.log("Using normal WebSocket");
		WebSocketType = "normal";
		b4j_ws = new WebSocket(fullpath);
	} else {	    
		if (isAndroidNative) {
		    console.log("Using ReconnectingWebSocket"); 
			WebSocketType = "reconnecting";
			b4j_ws = new ReconnectingWebSocket(fullpath);
			//b4j_ws = new RobustWebSocket(fullpath);
		} else {
			console.log("Using RobustWebSocket");
			WebSocketType = "robust";
			b4j_ws = new RobustWebSocket(fullpath);
			//b4j_ws = new ReconnectingWebSocket(fullpath);
		}
	}
	b4j_ws.addEventListener('message', function(event) {
        var ed = JSON.parse(event.data);
        if (ed.etype === "runmethod") {
            $(ed.id)[ed.method].apply($(ed.id), ed.params);
        } else if (ed.etype === "runmethodWithResult") {
            b4j_sendData($(ed.id)[ed.method].apply($(ed.id), ed.params));
        } else if (ed.etype === "setAutomaticEvents") {
            b4j_addAutomaticEvents(ed.data);
        } else if (ed.etype === "runFunction") {
            b4j_runFunction(ed.prop, ed.value);
        } else if (ed.etype === "runFunctionWithResult") {
            b4j_sendData(b4j_runFunction(ed.prop, ed.value));
        } else if (ed.etype === "eval") {
            b4j_eval(ed.value, ed.prop);
        } else if (ed.etype === "evalWithResult") {
            b4j_sendData(b4j_eval(ed.value, ed.prop));
        } else if (ed.etype === "alert") {
            window.alert(ed.prop);
        }
        
    });
	b4j_ws.addEventListener('open', function(event) {
		console.log("WebSocket opened - Page_Ready already send? " + PageReadyAlreadySend);
		if (reconnectTimeout) {
			clearInterval(reconnectTimeout);
		}
		//PageReadyAlreadySend = false; //???????????
		var el = document.body;
		var elIsLoading = document.getElementById('isloaderwrapper');
		if (elIsLoading) {
			if (!hasClass(elIsLoading, 'isloading')) {		
				if (!hasClass(el, 'loaded')) {
					addClass(el, 'loaded');
					var loader_el = document.getElementById('loader-wrapper');
					if (loader_el) {
						loader_el.parentElement.removeChild(loader_el);
					}
				} else {
					if (!PageReadyAlreadySend) {
						var loader_el = document.getElementById('loader-wrapper');
						if (!loader_el) {
							if (readyTimeout) {
								clearInterval(readyTimeout);
								//clearTimeout(readyTimeout);								
							}
							console.log("Scheduling Ready event")
							readyTimeout = setInterval(function() {
								if(document.readyState === 'complete') {
									clearInterval(readyTimeout);
									b4j_raiseEvent('page_parseevent', {'eventname':'page_ready','eventparams':''});
								}    
							}, 100);
							
							//readyTimeout = setTimeout(function() {b4j_raiseEvent('page_parseevent', {'eventname':'page_ready','eventparams':''});},500);
							//b4j_raiseEvent('page_parseevent', {'eventname':'page_ready','eventparams':''})						
						}
					}
				}
			} else {
				if (!hasClass(el, 'loaded') && !PageReadyAlreadySend) {
					var loader_el = document.getElementById('loader-wrapper');
					if (loader_el) {
						if (readyTimeout) {
							clearInterval(readyTimeout);
							//clearTimeout(readyTimeout);
						}
						console.log("Scheduling Ready event")
						readyTimeout = setInterval(function() {
							if(document.readyState === 'complete') {
								clearInterval(readyTimeout);
								b4j_raiseEvent('page_parseevent', {'eventname':'page_ready','eventparams':''});
							}    
						}, 100);
						//readyTimeout = setTimeout(function() {b4j_raiseEvent('page_parseevent', {'eventname':'page_ready','eventparams':''});},500);
						//b4j_raiseEvent('page_parseevent', {'eventname':'page_ready','eventparams':''})						
					}
				}
				
			}			
		} 
		var indicator = document.getElementById('pageconnectedindicator');
		if (indicator) {
			removeClass(indicator, 'indicatorinactive');
			addClass(indicator, 'indicatoractive');
		}
		
	});
	
	b4j_ws.addEventListener('close', function(event) {	
		//console.log("closed - navigator.onLine:" + navigator.onLine);
		if (readyTimeout) {
			clearInterval(readyTimeout);
			//clearTimeout(readyTimeout);
		}
		if (reconnectTimeout) {
			clearInterval(reconnectTimeout);
			//clearTimeout(readyTimeout);
		}
		var indicator = document.getElementById('pageconnectedindicator');
		if (indicator) {
			removeClass(indicator, 'indicatoractive');
			addClass(indicator, 'indicatorinactive');
		}	
		if (!navigator.onLine) {
			reconnectTimeout = setInterval(function() {
				if (navigator.onLine) {
					clearInterval(reconnectTimeout);
					if (WebSocketType=='robust') {
						b4j_ws.startReconnect(event);
					} 
				}    
			}, 1000);
		}
		//b4j_ws.startReconnect();
	});	

	
	/*
	if(window.performance) {		
		if(performance.navigation.type  == 1 ) {
           
        }
	};
	*/
	
	var isOnIOS = navigator.userAgent.match(/iPad/i)|| navigator.userAgent.match(/iPhone/i);
	var eventName = isOnIOS ? "pagehide" : "beforeunload";

	window.addEventListener(eventName, function (event) { 
		window.event.cancelBubble = true; // Don't know if this works on iOS but it might!
		b4j_raiseEvent('page_parseevent', {'eventname':'beforeunload','eventparams':''});
    } );
	
	/*
	window.onbeforeunload = function() { 
		b4j_raiseEvent('page_parseevent', {'eventname':'beforeunload','eventparams':''});
	};
	*/
	
}



